import getpass
import json
import os
import sys

from cdis_pipe_utils import df_util
from cdis_pipe_utils import pipe_util
from cdis_pipe_utils import time_util

def get_readgroup_str(readgroup_json_path, logger):
    json_data=open(readgroup_json_path).read()
    data = json.loads(json_data)
    data_list = [k + ':' + v for k,v in data.items()]
    if len(data_list) == 0: ## NEED TO TEST
        return None
    data_str = '\\t'.join(sorted(data_list)) # use \\t, so \t appears in @PG, and the @PG doesn't think there are 2 ID keys
    readgroup_str = '@RG\\t' + data_str
    return readgroup_str

def bwa_mem(uuid, fastq1_path, fastq2_path, reference_fasta_path, readgroup_json_path, thread_count, engine, logger):
    step_dir = os.getcwd()
    fastq1_name = os.path.basename(fastq1_path)
    fastq2_name = os.path.basename(fastq2_path)
    
    if fastq1_name.endswith('_1.fq.gz'):
        fastqbase_name = fastq1_name.replace('_1.fq.gz', '')
    elif fastq1_name.endswith('_1.fq'):
        fastqbase_name = fastq1_name.replace('_1.fq', '')
    else:
        logger.debug('unrecognized fastq file: %s' % fastq1_name)
        sys.exit(1)

    outbam = fastqbase_name + '.bam'
    
    rg_str = get_readgroup_str(readgroup_json_path, logger)

    home_dir = os.path.join('/home', getpass.getuser()) #cwltool sets HOME to /var/spool/cwl, so need to be explicit
    bwa_path = os.path.join(home_dir, '.local', 'bin', 'bwa')
    bwa_cmd = [bwa_path, 'mem', '-t ' + str(thread_count), '-T', '0', '-R ' + '"' + rg_str + '"', reference_fasta_path,
               '"' + fastq1_path + '"' , '"' + fastq2_path + '"']
    samtools_path = os.path.join(home_dir, '.local', 'bin', 'samtools')
    samtools_cmd = [samtools_path, 'view', '-Shb', '-o', '"' + outbam + '"', '-']
    shell_bwa_cmd = ' '.join(bwa_cmd)
    shell_samtools_cmd = ' '.join(samtools_cmd)
    shell_cmd = shell_bwa_cmd + ' | ' + shell_samtools_cmd
    output = pipe_util.do_shell_command(shell_cmd, logger)

    
    df = time_util.store_time(uuid, shell_cmd, output, logger)
    df['bam'] = outbam
    df['fastq1'] = fastq1_name
    df['fastq2'] = fastq2_name
    reference_fasta = os.path.basename(reference_fasta_path)
    df['reference_fasta'] = reference_fasta
    df['thread_count'] = thread_count
    unique_key_dict = {'uuid': uuid, 'bam': outbam, 'reference_fasta': reference_fasta,
                       'thread_count': thread_count}
    table_name = 'time_mem_bwa_mem_pe'
    df_util.save_df_to_sqlalchemy(df, unique_key_dict, table_name, engine, logger)
    
    return



#!/usr/bin/env python

import argparse
import logging
import os
import sys

import sqlalchemy

from cdis_pipe_utils import pipe_util

import tools.bwa_mem as bwa_mem

def is_nat(x):
    '''
    Checks that a value is a natural number.
    '''
    if int(x) > 0:
        return int(x)
    raise argparse.ArgumentTypeError('%s must be positive, non-zero' % x)

def check_for_table(engine, logger):
    if 'fastqc_data_Basic_Statistics' in engine.table_names():
        return
    else:
        logger.info('table "fastqc_data_Basic_Statistics" is missing')
        sys.exit(1)

def get_fastqc_measure(fastq_path, fastqc_db_path, measure_value, logger):
    engine_path = 'sqlite:///' + fastqc_db_path
    engine = sqlalchemy.create_engine(engine_path, isolation_level='SERIALIZABLE')
    fastq_name = os.path.basename(fastq_path)
    sql_text = 'select Value from fastqc_data_Basic_Statistics where fastq_name = "' + fastq_name \
               + '" and Measure = "' + measure_value +  '"'
    sql = sqlalchemy.text(sql_text)
    result = engine.execute(sql)
    result_all = result.fetchall()
    if len(result_all) != 1:
        logger('more/less than 1 result: %s' % result_all)
        sys.exit(1)
    else:
        row_value = result_all[0]
        if len(row_value) != 1:
            logger('more/less than 1 result: %s' % row_value)
            sys.exit(1)
        else:
            cell_value = row_value[0]
            if '-' in cell_value:
                str_value = cell_value.split('-')[-1]
                return str_value
            else:
                return cell_value
    return

def main():
    parser = argparse.ArgumentParser('bwa pe mapping')

    # Logging flags.
    parser.add_argument('-d', '--debug',
        action = 'store_const',
        const = logging.DEBUG,
        dest = 'level',
        help = 'Enable debug logging.',
    )
    parser.set_defaults(level = logging.INFO)

    # Required flags.
    parser.add_argument('-u', '--uuid',
                        required = True,
                        help = 'analysis_id string',
    )
    
    # Tool flags
    parser.add_argument('-f', '--reference_fasta_path',
                        required = True,
                        help = 'Reference fasta path.'
    )
    parser.add_argument('-1','--fastq1_path',
                        required = True
    )
    parser.add_argument('-2','--fastq2_path',
                        required = True
    )
    parser.add_argument('-r', '--readgroup_json_path',
                        required = True
    )
    parser.add_argument('--fastqc_db1_path',
                        required = True
    )
    parser.add_argument('--fastqc_db2_path',
                        required = True
    )
    parser.add_argument('-j', '--thread_count',
                        required = True,
                        type = is_nat,
                        help = 'Number of threads for execution.',
    )

    args = parser.parse_args()
    uuid = args.uuid
    reference_fasta_path = args.reference_fasta_path
    fastq1_path = args.fastq1_path
    fastq2_path = args.fastq2_path
    readgroup_json_path = args.readgroup_json_path
    fastqc_db1_path = args.fastqc_db1_path
    fastqc_db2_path = args.fastqc_db2_path
    thread_count = args.thread_count
    
    tool_name = 'bwa_pe'
    logger = pipe_util.setup_logging(tool_name, args, uuid)

    sqlite_name = uuid + '.db'
    engine_path = 'sqlite:///' + sqlite_name
    engine = sqlalchemy.create_engine(engine_path, isolation_level='SERIALIZABLE')

    hostname = os.uname()[1]
    logger.info('hostname=%s' % hostname)

    measure_value = 'Sequence length'
    fastq1_readlength = int(get_fastqc_measure(fastq1_path, fastqc_db1_path, measure_value, logger))
    logger.info('fastq1_readlength: %s' % fastq1_readlength)
    fastq2_readlength = int(get_fastq_readlength(fastq2_path, fastqc_db2_path, measure_value, logger))
    logger.info('fastq2_readlength: %s' % fastq2_readlength)
    fastq_readlength = max(fastq1_readlength, fastq2_readlength)

    measure_value = 'Encoding'
    fastq1_encoding = get_fastqc_measure(fastq1_path, fastqc_db1_path, measure_value, logger)
    logger.info('fastq1_encoding: %s' % fastq1_encoding)
    fastq2_encoding = get_fastq_readlength(fastq2_path, fastqc_db2_path, measure_value, logger)
    logger.info('fastq2_readlength: %s' % fastq2_encoding)
    if fastq1_encoding != fastq2_encoding:
        logger.info('unequal fastq encoding: %s != %s' % (fastq1_encoding, fastq2_encoding))

    if (fastq_readlength < 70) or (fastq1_encoding == 'Illumina 1.5') or (fastq1_encoding == 'Illumina-1.3'):
        bwa_aln.bwa_aln(uuid, fastq1_path, fastq2_path, fastq1_encoding,reference_fasta_path, readgroup_json_path,
                        thread_count, engine, logger)
    elif (fastq_readlength >= 70) and (fastq_encoding == 'Sanger / Illumina 1.9'):
        bwa_mem.bwa_mem(uuid, fastq1_path, fastq2_path, reference_fasta_path, readgroup_json_path, thread_count, engine, logger)
    else:
        logger.info('unrecognized fastq_encoding: %s' % fastq_encoding)


if __name__ == '__main__':
    main()
